package com.example.service.serviceImpl;

import com.alibaba.fastjson.JSON;
import com.example.db.dao.ArchiveDetailDao;
import com.example.db.dao.ArchiveMasterDao;
import com.example.db.dao.MessageSubordinateDao;
import com.example.service.SubordinateService;
import com.example.utils.CommonResult;
import com.example.vo.MessageDto;
import com.example.vo.MessageLog;
import com.example.vo.MessageSubordinateVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


/**
 * @ClassName SubordinateServiceImpl
 * @Description 处理消息接口
 * @Author linjj
 * @Date 2025/4/8 10:46
 * @Version 1.0
 */
@Service
@Slf4j
public class SubordinateServiceImpl implements SubordinateService {

    @Autowired
    private MessageSubordinateDao messageSubordinateDao;
    @Autowired
    private ArchiveMasterDao archiveMasterDao;
    @Autowired
    private ArchiveDetailDao archiveDetailDao;

    @Override
    public CommonResult<?> CompensateTask() {
        //查询哪些消息需要补偿0为未开始状态
        List<MessageSubordinateVo> list = messageSubordinateDao.getMessageSubordinate(0);
        if (CollectionUtils.isEmpty(list)) {
            log.info("当前没有需要补偿的数据");
            return CommonResult.success("当前没有需要补偿的数据");
        }
        for (MessageSubordinateVo messageSubordinate : list) {
            //判断消息中是否包含甲亢核医学PET-CT门诊不包含直接跳过不需要采集
            if (!messageSubordinate.getContentJson().contains("甲亢核医学PET-CT门诊")) {
                log.info("消息id为:" + messageSubordinate.getId() + "不包含甲亢核医学PET-CT门诊");
                //不是甲亢核医学PET-CT门诊的报告将状态改为5
                messageSubordinateDao.updateStatus(4, messageSubordinate.getId());
                continue;
            }
            //解析json返回患者基础信息
            MessageDto messageDto = parseMessage(messageSubordinate);
            if (messageDto == null) {
                //解释json信息失败将状态改为4
                messageSubordinateDao.updateStatus(4, messageSubordinate.getId());
                continue;
            }
            //采集失败将状态该为2,成功将状态改为1
            if (!compensate(messageDto)) {
                log.info("采集失败：" + messageSubordinate.getId());
                messageSubordinateDao.updateStatus(2, messageSubordinate.getId());
            } else {
                log.info("采集成功：" + messageSubordinate.getId());
                messageSubordinateDao.updateStatus(1, messageSubordinate.getId());
            }
        }
        return CommonResult.success("采集完成");
    }

    private Boolean compensate(MessageDto messageDto) {
        try {
            //获取下载地址
            String reportAddress = messageDto.getReportAddress();
            //查询该患者主键id
            List<String> ids = archiveMasterDao.getMasterIdByInpNoAndVisitId(messageDto.getInpNo(), messageDto.getVisitId());
            if (CollectionUtils.isEmpty(ids)) {
                return false;
            }
            //根据患者id查询文件表该文件的保存路径
            List<String> pdfPaths = archiveDetailDao.getPdfPath(ids.get(0), messageDto.getApplyId());
            //删除原地址文件
            if (!delPdf(pdfPaths)) {
                log.info("删除源文件失败：" + pdfPaths.get(0));
            }
            if (!DownloadFile(reportAddress, pdfPaths.get(0))) {
                return false;
            }
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * @Author: linjj
     * @Description: 删除原地址文件
     * @DateTime: 2025/4/8 16:26
     * @Params: [pdfPaths]
     * @Return java.lang.Boolean
     */
    private Boolean delPdf(List<String> pdfPaths) {
        try {
            //删除原地址文件
            File file = new File(pdfPaths.get(0));
            file.delete(); // 删除照片
        } catch (Exception e) {
            log.error("删除图像失败地址:" + pdfPaths.get(0) + e.getMessage(), e);
            return false;
        }
        return true;
    }

    /**
     * @Author: linjj
     * @Description: 解析json返回患者基础信息
     * @DateTime: 2025/4/8 12:03
     * @Params: [messageSubordinate]
     * @Return com.example.vo.MessageDto
     */
    private MessageDto parseMessage(MessageSubordinateVo messageSubordinate) {
        MessageDto messageDto = null;
        String id = messageSubordinate.getId();
        // 解析内容
        MessageLog messageLog = JSON.parseObject(messageSubordinate.getContentJson(), MessageLog.class);
        if (messageLog == null) {
            log.info("消息id为:" + id + "messageLog为空");
            return messageDto;
        }
        //获取患者基本信息
        String outJson = messageLog.getOutJson();
        if (outJson == null) {
            log.info("消息id为:" + id + "outJson为空");
            return messageDto;
        }
        messageDto = JSON.parseObject(outJson, MessageDto.class);
        return messageDto;
    }


    /**
     * @Author: linjj
     * @Description: 文件下载
     * @DateTime: 2025/4/08 14:50
     * @Params: [fileURL, savePath]
     * @Return java.lang.Boolean
     */
    private Boolean DownloadFile(String fileURL, String savePath) {
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();
            try (CloseableHttpResponse response = httpClient.execute(new HttpGet(fileURL))) {
                HttpEntity entity = response.getEntity();
                try (InputStream in = entity.getContent();
                     FileOutputStream fileOutputStream = new FileOutputStream(savePath)) {
                    byte[] buffer = new byte[1024];
                    int bytesRead;
                    while ((bytesRead = in.read(buffer)) != -1) {
                        fileOutputStream.write(buffer, 0, bytesRead);
                    }
                    return true;
                }
            }
        } catch (Exception e) {
            log.error("文件下载失败" + e.getMessage(), e);
            return false;
        }
    }
}
